@@ -137,7 +137,7 @@ DirectShowPlayerService::DirectShowPlayerService(QObject *parent)
     , m_pendingTasks(0)
     , m_executingTask(0)
     , m_executedTasks(0)
-    , m_taskHandle(::CreateEvent(0, 0, 0, 0))
+    , m_taskHandle(::CreateEvent(nullptr, FALSE, FALSE, nullptr))
     , m_eventHandle(0)
     , m_graphStatus(NoMedia)
     , m_stream(0)
@@ -209,8 +209,8 @@ QMediaControl *DirectShowPlayerService::requestControl(const char *name)
         if (!m_videoRendererControl && !m_videoWindowControl) {
             m_videoRendererControl = new DirectShowVideoRendererControl(m_loop);
 
-            connect(m_videoRendererControl, SIGNAL(filterChanged()),
-                    this, SLOT(videoOutputChanged()));
+            connect(m_videoRendererControl, &DirectShowVideoRendererControl::filterChanged,
+                    this, &DirectShowPlayerService::videoOutputChanged);
 
             return m_videoRendererControl;
         }
@@ -297,7 +297,8 @@ void DirectShowPlayerService::load(const QMediaContent &media, QIODevice *stream
     if (m_graph)
         releaseGraph();
 
-    m_resources = media.resources();
+    m_url = media.canonicalUrl();
+
     m_stream = stream;
     m_error = QMediaPlayer::NoError;
     m_errorString = QString();
@@ -312,11 +313,9 @@ void DirectShowPlayerService::load(const QMediaContent &media, QIODevice *stream
     m_dontCacheNextSeekResult = false;
     m_metaDataControl->reset();
 
-    if (m_resources.isEmpty() && !stream) {
+    if (m_url.isEmpty() && !stream) {
         m_pendingTasks = 0;
         m_graphStatus = NoMedia;
-
-        m_url.clear();
     } else if (stream && (!stream->isReadable() || stream->isSequential())) {
         m_pendingTasks = 0;
         m_graphStatus = InvalidMedia;
@@ -348,9 +347,6 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
 {
     IBaseFilter *source = 0;
 
-    QMediaResource resource = m_resources.takeFirst();
-    m_url = resource.url();
-
     HRESULT hr = E_FAIL;
     if (m_url.scheme() == QLatin1String("http") || m_url.scheme() == QLatin1String("https")) {
         static const GUID clsid_WMAsfReader = {
@@ -403,8 +399,6 @@ void DirectShowPlayerService::doSetUrlSource(QMutexLocker *locker)
             m_pendingTasks |= SetRate;
 
         m_source = source;
-    } else if (!m_resources.isEmpty()) {
-        m_pendingTasks |= SetUrlSource;
     } else {
         m_graphStatus = InvalidMedia;
 
@@ -526,9 +520,16 @@ void DirectShowPlayerService::doRender(QMutexLocker *locker)
                     } else {
                         locker->unlock();
                         HRESULT hr = graph->RenderEx(pin, /*AM_RENDEREX_RENDERTOEXISTINGRENDERERS*/ 1, 0);
-                        // Do not return an error if no video output is set yet.
-                        if (SUCCEEDED(hr) || !(m_executedTasks & SetVideoOutput)) {
+                        if (SUCCEEDED(hr)) {
                             rendered = true;
+                            m_error = QMediaPlayer::NoError;
+                        } else if (!(m_executedTasks & SetVideoOutput)) {
+                            // Do not return an error if no video output is set yet.
+                            rendered = true;
+                            // Remember the error in this case.
+                            // Handle it when playing is requested and no video output has been provided.
+                            m_error = QMediaPlayer::ResourceError;
+                            m_errorString = QString("%1: %2").arg(__FUNCTION__).arg(qt_error_string(hr));
                         } else if (renderHr == S_OK || renderHr == VFW_E_NO_DECOMPRESSOR){
                             renderHr = hr;
                         }
@@ -920,6 +921,16 @@ void DirectShowPlayerService::play()
 
 void DirectShowPlayerService::doPlay(QMutexLocker *locker)
 {
+    // Invalidate if there is an error while loading.
+    if (m_error != QMediaPlayer::NoError) {
+        m_graphStatus = InvalidMedia;
+        if (!m_errorString.isEmpty())
+            qWarning("%s", qPrintable(m_errorString));
+        m_errorString = QString();
+        QCoreApplication::postEvent(this, new QEvent(QEvent::Type(Error)));
+        return;
+    }
+
     if (IMediaControl *control = com_cast<IMediaControl>(m_graph, IID_IMediaControl)) {
         locker->unlock();
         HRESULT hr = control->Run();
@@ -951,7 +962,6 @@ void DirectShowPlayerService::pause()
 
     if (m_executedTasks & Render) {
         if (m_executedTasks & Stop) {
-            m_atEnd = false;
             if (m_seekPosition == -1) {
                 m_dontCacheNextSeekResult = true;
                 m_seekPosition = 0;
@@ -977,7 +987,8 @@ void DirectShowPlayerService::doPause(QMutexLocker *locker)
         control->Release();
 
         if (SUCCEEDED(hr)) {
-            if (IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking)) {
+            IMediaSeeking *seeking = com_cast<IMediaSeeking>(m_graph, IID_IMediaSeeking);
+            if (!m_atEnd && seeking) {
                 LONGLONG position = 0;
 
                 seeking->GetCurrentPosition(&position);
@@ -986,6 +997,7 @@ void DirectShowPlayerService::doPause(QMutexLocker *locker)
                 m_position = position / qt_directShowTimeScale;
             } else {
                 m_position = 0;
+                m_atEnd = false;
             }
 
             m_executedTasks |= Pause;
@@ -1616,7 +1628,7 @@ void DirectShowPlayerService::updateStatus()
         m_playerControl->updateStatus(QMediaPlayer::LoadingMedia);
         break;
     case Loaded:
-        if ((m_pendingTasks | m_executingTask | m_executedTasks) & (Play | Pause)) {
+        if ((m_executingTask | m_executedTasks) & (Play | Pause)) {
             if (m_buffering)
                 m_playerControl->updateStatus(QMediaPlayer::BufferingMedia);
             else
